// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWV: WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture OrigTex <string uiname="Original Texture";>;
sampler OrigSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (OrigTex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

texture DepthTex <string uiname="Depth Texture";>;
sampler DepthSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (DepthTex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

//convolution kernel
#define KERNEL_SIZE 7

float kernel[KERNEL_SIZE][KERNEL_SIZE] = {
      {1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0},
      {1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0},
      {1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0},
      {1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0},
      {1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0},
      {1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0},
      {1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0, 1.0/49.0}
};

/*
float kernel[KERNEL_SIZE][KERNEL_SIZE] = {
      {1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0},
      {1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0},
      {1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0},
      {1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0},
      {1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0, 1.0/25.0},
};
*/

float gamma = 1.0;

float TextureSize = 128;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float2 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = TexCd;
    
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float lp = 0;
    for (int x = 0; x < KERNEL_SIZE; x++) {
        for (int y = 0; y < KERNEL_SIZE; y++) {
           lp += kernel[x][y] * tex2D(DepthSamp, In.TexCd + float2(x - KERNEL_SIZE / 2, y - KERNEL_SIZE / 2) / TextureSize ).x;
        }
    }
    
    float d = lp - tex2D(DepthSamp, In.TexCd).x;
    float4 col = tex2D(OrigSamp, In.TexCd);
    if (d < 0.0) col += d * gamma;

    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique DepthDarkening
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}
